#ifndef _COMMUNITY_H_
#define _COMMUNITY_H_

#include <list>
#include "types.h"
#include "ordered_list.h"
#include <set>
#include <vector>

class Community {
  private:
	static int next_available_number;
	int number;

	std::vector<int> vertices;
	unsigned int sumDegrees;
	unsigned int sumEdges;
	unsigned int size;
	unsigned int maxDegree;
	unsigned int maxShellIndex;

	void setCommunityCutWithMap(Community* c, int n = 1);
	void removeCommunityCutWithMap(Community* c, int n = 1);
	void addCommunityCut(Community* c, unsigned int n = 1);
	void removeCommunityCut(Community* c, unsigned int n = 1);
	bool haveMap; //Map instead of vector

	//For graphics
	float rhoCoordinate;
	float phiCoordinate;
	float xCoordinate;
	float yCoordinate;
	float ratioAbstract;
	float ratioReal;

  public:
	Community();
	~Community();

	std::vector<Community*> community_cuts1; //Neighbour community
	std::vector<unsigned int> community_cuts2; //Cut with the neighbour community
	std::vector<unsigned int> community_cuts3; //MY POSITION IN THE NEIGHBOUR COMMUNITY'S VECTOR OF NEIGHBOUR COMMUNITIES
	std::set<bestNeighbour*, Compare> community_cuts;

	inline int getNumber() { return number; }
	bool removed;

	inline int getSize() const { return size; }

	inline void addVertex(int v) { vertices.push_back(v); }
	void addCommunityWithMap(Community* C, int prevcut);
	void addCommunity(Community* C, unsigned int prevcut, float t);
	void renew();
	void removeCommunity();
	//void removeVertex(Vertex* v);

	bool hasNeighbour(Community* c);
	void addNeighbour(Community* c);

	inline std::vector<int>& getVertices() { return vertices; }
	inline unsigned int tDegrees() { return sumDegrees; }
	inline unsigned int tEdges() { return sumEdges; }
	inline unsigned int getMaxDegree() { return maxDegree; }

	void cambiarVectorXMap();

	//Recomputes parameters, probably because vertices have changed
	void recompute();

/*
	inline int cut(Community* c) {
		std::map<Community*, int>::iterator it = community_cuts.find(c);
		if (community_cuts.find(c)!=community_cuts.end())
			return it->second;
		return 0; };
*/

	inline std::set<bestNeighbour*, Compare>& getCommunityCuts() {return community_cuts; };
	void printCommunity();

	void computeBestNeighbour();

	//For graphics
	inline float getRhoCoordinate() { return rhoCoordinate;}
	inline float getPhiCoordinate() { return phiCoordinate;}
	inline float getX() { return xCoordinate;}
	inline float getY() { return yCoordinate;}
	inline float getRatioAbstract() { return ratioAbstract;}
	inline float getRatioReal() { return ratioReal;}
	inline unsigned int getMaxShellIndex() { return maxShellIndex;}
	inline void setRhoCoordinate(float rho) { rhoCoordinate = rho;}
	inline void setPhiCoordinate(float phi) { phiCoordinate = phi;}
	inline float setX(float x) { xCoordinate = x;}
	inline float setY(float y) { yCoordinate = y;}
	inline void setRatioAbstract(float r) { ratioAbstract = r;}
	inline void setRatioReal(float r) { ratioReal = r;}
	inline void setMaxShellIndex(unsigned int si) { maxShellIndex = si;}

	inline std::vector<Community*> getNeighbourCommunities() { return community_cuts1; }
	inline std::vector<unsigned int> getNeighbourCommunityCuts() { return community_cuts2; }
	//For use by solution_suborthogonal
	int selected;
	float temp1, temp2;
	bestNeighbour* bn;
};

#endif

