#ifndef _COMPONENT_H_
#define _COMPONENT_H_

#include <list>
#include "vertex.h"
#include "community.h"
#include "graph.h"

class Vertex;
class Graph;
class Community;

extern float central_core_ratio;

class Component {
  private:

	std::list<std::list<Vertex*>* > clusters;
	//All vertices in this component (including vertices in internal components)
	std::list<Vertex*> all_vertices;
	std::list<Component*> components;

	Component* parentComponent;

	int size; //Vertices in this component (including ones in inner components)
	int shellCardinal; //Vertices in this component clusters with this shellIndex
	int shellIndex;

//	float central_core_ratio;
	float rho, phi, xCoord, yCoord, u;

	//Auxiliar functions for computeComponents
	void neighboursClusterPropagation(Vertex* v, std::list<Vertex*>* cluster);
	void neighboursComponentPropagation(Vertex* v, Component* component, std::list<Vertex*>* cluster);

	void computeComponents(float &c, float maxShellIndex, float minShellIndex, float maxDegree);
	
  public:
	Component(int m_shellIndex, Component* m_parentComponent);

	void computeComponents(std::list<Vertex*>* vertices, float maxShellIndex, float minShellIndex, float maxDegree);
	void printComponent();

	void placeInCircularSector(float ratio, float alfa, int n, int total, float random, float& rho, float& phi);

	void findCommunities(std::list<Community*>& communities, Graph* partial_graph);

	inline int getAmountComponents() {return components.size();}
	inline int getSize() {return size;}
	inline std::list<Component*>& getComponents() {return components;}
	inline Component* getParentComponent() {return parentComponent;}
	inline std::list<Vertex*>& getVertices() {return all_vertices;}
	inline std::list<std::list<Vertex*>* >& getClusters() { return clusters; };

	void buildClustersMap(std::map<int, std::list<std::list<Vertex*>* > >& clustersMap);

	inline void setRho(float r) { rho = r;}
	inline void setPhi(float p) { phi = p;}
	inline void setX(float x) { xCoord = x;}
	inline void setY(float y) { yCoord = y;}
	inline void setU(float ui) { u = ui;}
	inline float getRho() { return rho;}
	inline float getPhi() { return phi;}
	inline float getX() { return xCoord;}
	inline float getY() { return yCoord;}
	inline float getU() { return u;}
	inline float getCentral_core_ratio() { return central_core_ratio; }
	inline float getShellIndex() { return shellIndex;}

	int getBrotherComponentsCardinal();
	int getPreviousComponentsCardinal();
	inline bool hasParentComponent() { if (parentComponent!=NULL) return true; else return false;}

};

#endif

