#include "graph.h"
#include <iostream>
#include <algorithm>

Graph::Graph(bool m_simple) {
	simple = m_simple;
	totalVertices = 0;
	total2Edges = 0;
	iq=0;
	sumSquareDegrees = 0;
	maxDegree = -1;
	maxStrength = 0.0;
	maxVertex=-1;
}

void Graph::initialize(unsigned int maxVertex) {
	existent_vertices.resize(maxVertex+1);
	int n = existent_vertices.size();
	for (int i=0;i<n;i++) {
		Community* va = new Community();
		v.push_back(va);
		existent_vertices[i] = NULL;
	}
}

void Graph::addName(int i, std::string n) {
	if (vertices_map.find(i)!=vertices_map.end())
		vertices_map[i]->setName(n);
}

void Graph::addEdge(unsigned int n1, unsigned int n2) {
	//If a vertex does not exist, it's created
	std::map<int, Vertex*>::iterator it1, it2;
	Community *v1, *v2;
	Vertex *vt1, *vt2;
	it1 = vertices_map.find(n1);
//	if (it1==vertices_map.end()) {
	if (existent_vertices[n1]==NULL) {
		Vertex* vt = new Vertex(n1);
		v[iq]->addVertex(n1);
		existent_vertices[n1] = v[iq];
		totalVertices++;
		vertices.push_back(vt);
		vertices_map[n1] = vt;
		v1 = v[iq];
		vt1 = vt;
		if (n1>maxVertex)
			maxVertex=n1;
		iq++;
	}
	else {
		vt1 = it1->second;
		v1 = existent_vertices[n1];
	}

	it2 = vertices_map.find(n2);
//	if (it2==vertices_map.end()) {
	if (existent_vertices[n2]==NULL) {
		Vertex* vt = new Vertex(n2);
		v[iq]->addVertex(n2);
		existent_vertices[n2] = v[iq];
		totalVertices++;
		vertices.push_back(vt);
		vertices_map[n2] = vt;
		v2 = v[iq];
		vt2 = vt;
		iq++;
	}
	else {
		vt2 = it2->second;
		v2 = existent_vertices[n2];
	}

	if ((!simple)||(v1->hasNeighbour(v2) == false)) {
		v1->addNeighbour(v2);
		vt1->addNeighbour(vt2);
		total2Edges++;
		if (n1!=n2) {
			v2->addNeighbour(v1);
			vt2->addNeighbour(vt1);
			total2Edges++;
		}

		if (vt1->getDegree() > maxDegree)
			maxDegree = vt1->getDegree();	
		if (vt2->getDegree() > maxDegree)
			maxDegree = vt2->getDegree();	

		sumSquareDegrees += 2*vt1->getDegree() - 1 + 2*vt2->getDegree() - 1;

		//Communities
//		if (v1->getCommunity() != NULL) {
			//v2->addCommunityCut(v1->getCommunity());
//			v2->getCommunity()->addCommunityCut(v1->getCommunity());
//		}
//		if (v2->getCommunity() != NULL) {
			//v1->addCommunityCut(v2->getCommunity());
//			v1->getCommunity()->addCommunityCut(v2->getCommunity());
//		}

	}
}

void Graph::buildDegreesList() {
	std::list<Vertex*>::iterator it;
	for (it=vertices.begin();it!=vertices.end();it++) {
		degreesList[(*it)->getDegree()].push_back(*it);
	}
}

void Graph::printGraph() {
	std::cout << "Graph BEGIN" << std::endl;
	std::vector<Vertex*>* pNeighbours;
	std::list<Vertex*>::iterator it;
	std::vector<Vertex*>::iterator it2;
	for (it=vertices.begin();it!=vertices.end();it++) {
		std::cout << (*it)->getNumber() << " --> ";
		pNeighbours = (*it)->getNeighbours();
		for (it2=pNeighbours->begin();it2!=pNeighbours->end();it2++) {
			std::cout << (*it2)->getNumber() << " ";
		}
		std::cout << std::endl;
	}
	std::cout << "Graph END" << std::endl;
}

void Graph::printDegreesList() {
	std::cout << "Degrees List BEGIN" << std::endl;
	std::map<int, std::list<Vertex*> >::iterator it;
	std::list<Vertex*>::iterator it2;
	for (it=degreesList.begin();it!=degreesList.end();it++) {
		std::cout << it->first << ": ";
		for (it2=it->second.begin();it2!=it->second.end();it2++)
			std::cout << (*it2)->getNumber() << " ";
		std::cout << std::endl;
	}
	std::cout << "Degrees List END" << std::endl;
}

//Adds a list of vertices to the graph. Those vertex neighbours that don't belong to the graph, are removed from the list of neighbours. The vertices are copies from the parameter. The vertices are returned.
std::vector<Vertex*> Graph::addVertices(std::vector<Vertex*>& vertices) {

	std::vector<Vertex*> new_vertices;
	Vertex* v;

	std::vector<Vertex*>::iterator it;
	for (it=vertices.begin();it!=vertices.end();it++) {
		v = addVertex(*it);
		new_vertices.push_back(v);
	}
	return new_vertices;
}

std::list<Vertex*> Graph::addVertices(std::list<Vertex*>& vertices) {

	std::list<Vertex*> new_vertices;
	Vertex* v;

	std::list<Vertex*>::iterator it;
	for (it=vertices.begin();it!=vertices.end();it++) {
		v = addVertex(*it);
		new_vertices.push_back(v);
	}
	return new_vertices;
}

//Adds a vertex to the graph. Those vertex neighbours that don't belong to the graph, are removed from the list of neighbours. The vertex is a copy from the parameter. The vertex is returned.
Vertex* Graph::addVertex(Vertex* v) {
	Vertex* vertex = new Vertex(v);
	totalVertices++;
	vertices.push_back(vertex);
	vertices_map[v->getNumber()] = vertex;
	if (v->getNumber()>maxVertex)
		maxVertex=v->getNumber();

	std::vector<Vertex*>::iterator it;
	std::map<int, Vertex*>::iterator it2;

	for (it=v->getNeighbours()->begin();it!=v->getNeighbours()->end();it++) {
		//If the neighbour exists in this graph
		it2 = vertices_map.find((*it)->getNumber());
		if (it2!=vertices_map.end()) {
			(it2->second)->addNeighbour(vertex);
			vertex->addNeighbour(it2->second);
			total2Edges++;
			if (vertex->getNumber()!=v->getNumber())
				total2Edges++;
			sumSquareDegrees += 2*(it2->second)->getDegree() - 1 + 2*vertex->getDegree() - 1;

			if ((it2->second)->getDegree() > maxDegree)
				maxDegree = (it2->second)->getDegree();	

			//Communities
			if ((it2->second)->getCommunity() != NULL) {
				//vertex->addCommunityCut((it2->second)->getCommunity());
				//vertex->getCommunity()->addCommunityCut((it2->second)->getCommunity());
			}
		}
	}
	return vertex;
}

bool Graph::computeDiameter2() {
	unsigned int ov = (pow(2, 31) - 1);
	ov += pow(2, 31);
	unsigned int maxVertex = vertices.size();

	//Constructing vectors
	std::vector<unsigned int> v;
	v.resize((maxVertex-1)/32 + 1);
	for (unsigned int i=0;i<(maxVertex-1)/32 + 1;i++)
		v[i] = 0;
	if ((maxVertex) % 32 != 0) {
		v[(maxVertex-1)/32] = ov - (unsigned int)round(pow(2, maxVertex % 32) - 1);
	}

	std::vector<std::vector<unsigned int> > vl, vl2;
	for (unsigned int i=0;i<maxVertex;i++)
		vl.push_back(v);
	for (unsigned int i=0;i<maxVertex;i++)
		vl2.push_back(v);
	
	std::list<Vertex*>::iterator v_it, it;
	std::vector<Vertex*>::iterator nb_it;

	unsigned int i=0;
	for (v_it=vertices.begin();v_it!=vertices.end();v_it++) {
		Vertex* v = *v_it;
		unsigned int posArray = i/32;
		vl[i][posArray] = vl[i][posArray] | (unsigned int)round(pow(2, i % 32));
		for (nb_it=v->getNeighbours()->begin();nb_it!=v->getNeighbours()->end();nb_it++) {
			Vertex* n = *nb_it;
			unsigned int number=0;
			for (it=vertices.begin();*it!=n;it++)
				number++;
			unsigned int posArray = number/32;
			vl[i][posArray] = vl[i][posArray] | (unsigned int)round(pow(2, number % 32));
		}
		i++;
	}

	//ORs
	i=0;
	for (v_it=vertices.begin();v_it!=vertices.end();v_it++) {
		Vertex* v = *v_it;
		for (unsigned int j=0;j<=(maxVertex-1)/32;j++)
			vl2[i][j] = vl[i][j];

		for (nb_it=v->getNeighbours()->begin();nb_it!=v->getNeighbours()->end();nb_it++) {
			Vertex* n = *nb_it;
			unsigned int number=0;
			for (it=vertices.begin();*it!=n;it++)
				number++;
			for (unsigned int j=0;j<=(maxVertex-1)/32;j++)
				vl2[i][j] = vl2[i][j] | vl[number][j];
		}
		//Prunsigned intf.prunsigned intf "Vector %d: %x\n" n#getNumberInFile (!vl2).(i).(0);
		i++;
	}

	//Verifying
	unsigned int result = 2;
	for (unsigned int i=0;i<=maxVertex-1;i++)
		for (unsigned int j=0;j<=(maxVertex-1)/32;j++) {
			if (vl2[i][j] != ov) {
				//std::cout << "i,j,Diff: " << i << " " << j << ov - vl2[i][j] << std::endl;
				result = 3;
			}
		}

	return (result==2);
}

//TODO
/**method endGraph =
		(**Sorting of strengths list, if it's a weighted graph*)
		if (Parameters.parameters#getWeighted = true)&&(Parameters.parameters#getMultigraph = false) then
		begin
			for i= 0 to (List.length vertices * arraySize -1) do
				let posList = i / arraySize in
				if (List.nth vertices posList).(i mod arraySize)#isVertex then
				begin
					let strength = (List.nth vertices posList).(i mod arraySize)#getStrength in
					strengths <- strength::strengths;
					if strength > maxStrength then
						maxStrength <- strength;
				end
			done;
			strengths <- List.sort (fun a b -> (if a > b then 1 else -1)) strengths;

			(**let rec printStrengthsList w = 
			match w with
				[] -> ()
				| a :: l ->
					Printf.printf "%f " a;
					printStrengthsList l;
			in
			printStrengthsList strengths;
			Printf.printf "\n";*)
			(**p-function is built*)
			self#buildPFunction;
		end;

		(**p-function is applied to every vertex (every vertex applies it to each of its edges*)
		for i= 0 to (List.length vertices * arraySize -1) do
			let posList = i / arraySize in
			if (List.nth vertices posList).(i mod arraySize)#isVertex then
			begin
				let p = (List.nth vertices posList).(i mod arraySize)#getP pFunction (-2) in
				if p > maxPValue then
					maxPValue <- p;
			end
			
		done;
*/


