#ifndef _GRAPH_H_
#define _GRAPH_H_

#include "vertex.h"
#include <map>
#include <list>
#include <vector>

class Vertex;

class Graph {
  private:

	int iq;
	std::vector<Community*> v;

	int sumSquareDegrees;

	bool simple;

	int maxVertex;

	float rhoCoordinate;
	float phiCoordinate;
	float ratio;

  protected:
	//Fast access to vertices
	//std::map<int, Vertex*> vertices_map;
	std::vector<Community*> existent_vertices;

	unsigned int totalVertices, total2Edges;

  protected:
	//Fast access to vertices
	std::map<int, Vertex*> vertices_map;

	std::list<Vertex*> vertices;

	//Maximum degree of a vertex. Initially set to -1 (unknown)
	int maxDegree;
		
	float maxStrength;

	//Map of vertices of the same degree
	std::map<int, std::list<Vertex*> > degreesList;

  public:
	Graph(bool m_simple = true);

	void initialize(unsigned int maxVertex);

	void addEdge(unsigned int n1, unsigned int n2);

	void buildDegreesList();

	void printGraph();
	void printDegreesList();

	inline float getRhoCoordinate() { return rhoCoordinate;};
	inline float getPhiCoordinate() { return phiCoordinate;}
	inline float getRatio() { return ratio;}
	inline void setRhoCoordinate(float rho) { rhoCoordinate = rho;};
	inline void setPhiCoordinate(float phi) { phiCoordinate = phi;}
	inline void setRatio(float r) { ratio = r;}

	//Adds a list of vertices to the graph. Those vertex neighbours that don't belong to the graph, are removed from the list of neighbours. The vertices are copies from the parameter. The vertices are returned
	std::vector<Vertex*> addVertices(std::vector<Vertex*>& vertices);
	std::list<Vertex*> addVertices(std::list<Vertex*>& vertices);

	//Adds a vertex to the graph. Those vertex neighbours that don't belong to the graph, are removed from the list of neighbours. The vertex is a copy from the parameter. The vertex is returned.
	Vertex* addVertex(Vertex* v);

	inline std::list<Vertex*>& getVertices() { return vertices;}
	inline std::map<int, Vertex*>& getMapIntVertices() { return vertices_map;}

	inline Vertex* getVertex(int i) { return vertices_map[i];}

	//Returns amount of vertices
	inline int tVertices() { return totalVertices; }
	//Returns amount of edges
	inline int tEdges() { return total2Edges; }

	inline int tDegrees() { return total2Edges; }
	inline int tSqDegrees() { return sumSquareDegrees; }

	inline int getMaxDegree() { return maxDegree;}

	inline float getMaxStrength() { return maxStrength; }

	inline int getMaxVertex() { return maxVertex; }

	void addName(int i, std::string n);

	bool computeDiameter2();

};

#endif

