#include "graphics.h"
#include "parameters.h"
#include "uniform.h"
#include "component.h"
#include "espiral.h"

Uniform uniform(0,1);

float S = 1.0; //Constante de ajuste del tamaño de los nodos

Graphics::Graphics(Engine* e) {
	engine = e;
	xMeshSize = 1;
	yMeshSize = 1;
}

tColor Graphics::interpolate(tColor c1, float alfa1, tColor c2) {
	tColor c;
	c.r = c1.r * (1.0 - alfa1) + c2.r * (alfa1);
	c.g = c1.g * (1.0 - alfa1) + c2.g * (alfa1);
	c.b = c1.b * (1.0 - alfa1) + c2.b * (alfa1);
	return c;
}	

bool par(int n) {
	if (n%2==0)
		return true;
	return false;
}

tColor Graphics::computeHostColor(int maxShellIndex, int shellIndex) {
	std::vector<std::pair<tColor, float> > colorList;
	
	if (parameters.getColor()=="col")
		colorList = rainbow;
	else
		colorList = blackwhite;

	int colorsNumber = colorList.size();

	float position;
	if (parameters.getColor()=="col")
		position = (float)(shellIndex - 1) / (maxShellIndex - 1);
	else if (parameters.getColor()=="bw")
		position = (float)(shellIndex - 1) / (maxShellIndex - 1);
	else if ((parameters.getColor()=="bwi")&&(par(shellIndex + maxShellIndex)))
		position = ((float)(maxShellIndex - 1) + (float)(shellIndex - 1)) / (2.00 * (maxShellIndex - 1));
	else 
		position = (float)(shellIndex - 1) / (2.00 * (maxShellIndex - 1));

	//Color calculation
	tColor hostColor;
	if (maxShellIndex==1)
		hostColor = red;
	else {
		std::pair<tColor, float> p1 = colorList[0];
		std::pair<tColor, float> p2 = colorList[colorsNumber-1];
		if (position < p1.second)
			hostColor = p1.first;
		else if (position > p2.second)
			hostColor = p2.first;
		else {
			int index = 0;
			bool enc = false;
			
			while (enc==false) {
				p1 = colorList[index];
				p2 = colorList[index + 1];
				if ((p1.second<=position)&&(position<=p2.second))
					enc = true;
				else
					index += 1;
			}

			p1 = colorList[index];
			p2 = colorList[index + 1];
			float alfa1 = (float)(position - p1.second) / (p2.second - p1.second);
			hostColor = interpolate(p1.first, alfa1, p2.first);
		}
	}
	
	//End of color calculation
	if (parameters.getColor()=="col") {
		float lumin = 0.7 + 0.5 * (float)(shellIndex%2);
		tColor color2;
		color2.r = lumin * hostColor.r;
		color2.g = lumin * hostColor.g;
		color2.b = lumin * hostColor.b;
		return color2;
	}
	return hostColor;
}
		
float Graphics::computeHostRatio(int maxDegree, float strength, int degree, bool weighted) {
	if (weighted==false)
		return (0.0007 + 0.029 * (log((float)(1 + degree)) / log(maxDegree)) * 15);
//TODO
//	return (0.0007 + 0.029 * (log(1.0 + strength) / log(n.getMaxStrength()))) * (n.getMaxShellIndex() + 1.0 - n.getMinShellIndex());
}

void translate(Vertex* host, float rho, float phi, float ratio, float fullRatio) {
	//TODO: Trasladar vértice
	host->setX(host->getX() * ratio + rho * cos(phi));
	host->setY(host->getY() * ratio + rho * sin(phi));
//	host->setRatio(host->getRatio() * ratio / fullRatio);
}

void Graphics::drawNode(Vertex* host, float u) {
		engine->addHost(host->getX(), host->getY(), -(float)host->getShellIndex()/10, (host->getRatio() * u), host->getColor(), host->getBorderColor());
}

void Graphics::drawText(Vertex* host, float u, tColor textColor, float ppcV, float ppcH, float ratio, Network* n) {
	int i, ni;

	//Adding node's name

	bool hasName = true;
	std::string nodeName;
	std::stringstream strm;

	if (host->getName() != "") {
		nodeName = host->getName();
	}
	else {
		if (parameters.getInputNodeNamesFile()=="0") {
			strm << host->getNumber();
			nodeName = strm.str();
		}
		else {
			hasName = false;
			nodeName = "";
		}
	}

	if (hasName==true) {
		//if host#getName <> "" then
		//Printf.printf "Nodo %s\n" host#getName;
		float large = parameters.getU() * ratio;
		float size = 0.05 * large * 800.0 * parameters.getFontZoom() / parameters.getWidthResolution();
		//engine#addText host#getName textColor size ((host#getX +. 0.08) /. size) (host#getY /. size);
	
		float xCoordRight = (host->getX() + 1.2 * host->getRatio()) / size;
		float yCoordRight = host->getY() / size;
		tLetra ip1 = { xCoordRight, yCoordRight, 1, 0};
								
		float xCoordLeft = (host->getX() - 1.2 * host->getRatio()) / size;
		float yCoordLeft = host->getY() / size;
		tLetra ip2 = { xCoordLeft, yCoordLeft, -1, 0};

		float xCoordUp = host->getX() / size;
		float yCoordUp = (host->getY() + 1.2 * host->getRatio()) / size;
		tLetra ip3 = { xCoordUp, yCoordUp, 1, 0};
		tLetra ip4 = { xCoordUp, yCoordUp, -1, 0};
	
		float xCoordDown = host->getX() / size;
		float yCoordDown = (host->getY() - 1.2 * host->getRatio()) / size;
		tLetra ip5 = { xCoordDown, yCoordDown, 1, 0};
		tLetra ip6 = { xCoordDown, yCoordDown, -1, 0};
	
		std::vector<tLetra> meshPosVector;
		meshPosVector.push_back(ip1);
		meshPosVector.push_back(ip2);
		meshPosVector.push_back(ip3);
		meshPosVector.push_back(ip4);
		meshPosVector.push_back(ip5);
		meshPosVector.push_back(ip6);
	
		//let nodeName = (Printf.sprintf "%d" host#getNumber) in
		int chars = nodeName.length();
	
		int overlaps[6] =  {0, 0, 0, 0, 0, 0};
		int overlapsi;
	
		//For each position
		for (ni=0;ni<=meshPosVector.size();ni++) {
			int xMesh = round(((float)xMeshSize / 2) + (meshPosVector[ni].xi * size / (2.0 * large)) * xMeshSize);
			int yMesh = round(((float)yMeshSize / 2) + (meshPosVector[ni].yi * size / (2.0 * large)) * yMeshSize);
		
			//Printf.printf "n=%d xMesh: %d yMesh: %d\n" n xMesh yMesh;

			float dir = meshPosVector[ni].dirxi;
			//For each character
			for (i=0;i<=chars;i++) {
			
				if (((xMesh+dir*i) < xMeshSize)&&((xMesh+dir*i)>0)&&(yMesh < yMeshSize)&&(yMesh>0)) {
				
					if (namesMesh[xMesh+dir*i][yMesh] != 0) {
						overlapsi = namesMesh[xMesh+dir*i][yMesh];
						overlaps[ni] += overlapsi;
					}
				} else
					overlaps[ni] += 1;
			
				if (((xMesh+dir*i) < xMeshSize)&&((xMesh+dir*i)>0)&&(yMesh + 1 < yMeshSize)&&(yMesh + 1 >0)) {
					if (namesMesh[xMesh+dir*i][yMesh + 1] != 0) {
						overlapsi = namesMesh[xMesh+dir*i][yMesh + 1];
						overlaps[ni] += overlapsi;
					}
				} else
					overlaps[ni] += 1;
			
				if (((xMesh+dir*i) < xMeshSize)&&((xMesh+dir*i)>0)&&(yMesh - 1 < yMeshSize)&&(yMesh - 1 >0)) {
				
					if (namesMesh[xMesh+dir*i][yMesh - 1] != 0) {
						overlapsi = namesMesh[xMesh+dir*i][yMesh - 1];
						overlaps[ni] += overlapsi;
					}
				} else
					overlaps[ni] += 1;
			}
		}
	
		//Printf.printf " [%d %d %d %d %d %d] \n" !overlaps.(0) !overlaps.(1) !overlaps.(2) !overlaps.(3) !overlaps.(4) !overlaps.(5);
	
		int minPos = 0;
		for (ni=1;ni<=5;ni++) {
			if (overlaps[ni] < overlaps[minPos])
				minPos = ni;
		}
	
		int xMesh = round(((float)xMeshSize / 2) + (meshPosVector[minPos].xi * size / (2.0 * large)) * xMeshSize);
		int yMesh = round(((float)yMeshSize / 2) + (meshPosVector[minPos].yi * size / (2.0 * large)) * yMeshSize);

		int dir =  meshPosVector[minPos].dirxi;
		//Printf.printf "pos %d\n" !minPos;
	
		//If adding this text doesn't result in a great superposition
		if ((overlaps[minPos] <= 1)&&((float)overlaps[minPos]/chars <= 0.4)) {
			if (dir==1) {
				engine->addText(nodeName, textColor, size, meshPosVector[minPos].xi, meshPosVector[minPos].yi, "left");
			} else {
				engine->addText(nodeName, textColor, size, (meshPosVector[minPos].xi - (chars + 1) * ((float)ppcH / parameters.getWidthResolution()) * 2.0 * large / size), meshPosVector[minPos].yi, "right");
			}
			
			//Marking mesh
			for (i=0;i<=chars;i++) {
				if (((xMesh+dir*i) < xMeshSize)&&((xMesh+dir*i)>0)&&(yMesh < yMeshSize)&&(yMesh>0))
					namesMesh[xMesh+dir*i][yMesh] += 1;
			}
		}
	}

}

void Graphics::drawEdge(Vertex* host, float u, float ratio, Network* n, bool drawInternalCommunityEdges, bool drawExternalCommunityEdges) {
	int i, ni;

	Uniform unif(-15, 15);

	//Adding edge
	Vertex* ngb;
	std::vector<Vertex*>::iterator ngb_it;
	for (ngb_it=host->getNeighbours()->begin();ngb_it!=host->getNeighbours()->end();ngb_it++) {
		ngb = *ngb_it;
//			std::cout << "Eje de " << host->getNumber() << " con " << ngb->getNumber() << std::endl;
		//TODO
		//p = peso

		float p = 1;
		//Probability factor
		float max2 = parameters.getPercentOfEdges();
		float z = 2 * 15;
		if ((float)parameters.getMinEdges() / n->tEdges()>max2)
			max2 = (float)parameters.getMinEdges() / n->tEdges();
		float theValue = uniform.getValue();
		if (theValue < max2)  {
			//numberOfEdges += 1;
			//To avoid selecting the same edge when host n is analyzed
			if (host->getNumber() > ngb->getNumber()) {
				tColor ngbColor = ngb->getColor();
				tColor hColor = host->getColor();
				if (n->getVertex(host->getNumber())->getCommunity() == n->getVertex(ngb->getNumber())->getCommunity()) {
					if (drawInternalCommunityEdges)
						engine->addCylinder(host->getX(), host->getY(), z, ngb->getX(), ngb->getY(), z, ngbColor, parameters.getOpacity(), (p * 0.00176 * (15)));
				}
				else {
					if (drawExternalCommunityEdges)
						engine->addCylinder(host->getX(), host->getY(), z, ngb->getX(), ngb->getY(), z, white, parameters.getOpacity() / 3, (p * 0.00176 * (15)));
				}
				
			}
		}
	}
}

void Graphics::generateNetworkFile(std::vector<Network*> nv) {
}

typedef Community* CommunityPtr;

bool communitiesSizeOrder(const CommunityPtr& c1, const CommunityPtr& c2) { 
	//Comparing external edges...
	return (c1->tDegrees() - c1->tEdges() > c2->tDegrees() - c2->tEdges());
}

bool communitiesInternalSizeOrder(const CommunityPtr& c1, const CommunityPtr& c2) { 
	return (c1->tEdges() > c2->tEdges());
}

class CompareCommunitiesSize {
  public:
	bool operator()(const CommunityPtr& c1, const CommunityPtr& c2) { return communitiesSizeOrder(c1, c2);};
};

class CompareCommunitiesInternalSize {
  public:
	bool operator()(const CommunityPtr& c1, const CommunityPtr& c2) { return communitiesInternalSizeOrder(c1, c2);};
};

void Graphics::compute_sizes_for_real_communities(std::list<Community*> lc, Network* n) {
	std::list<Community*>::iterator lc_it;
	Vertex* h;
	std::vector<int>::iterator vertex_id_it;
	//Calculo los tamaños
	for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
		Community* c = *lc_it;
		float area = 0;
		for (vertex_id_it=c->getVertices().begin();vertex_id_it!=c->getVertices().end();vertex_id_it++) {
			int i = *vertex_id_it;
			h = n->getVertex(i);
			//std::cout << "Core " << h->getShellIndex() << std::endl;
			float hostRatio = computeHostRatio(n->getMaxDegree(), (h->getStrength()), (h->getDegree()), (parameters.getWeighted() && (!parameters.getMultigraph())));
			h->setRatio(hostRatio);
			area += hostRatio * hostRatio;
		}
//		std::cout << "Nodos: " << c->getSize() << std::endl;
//		c->setRatio(sqrt((float)c->getSize()*50/n->tVertices()));
		c->setRatioReal(S*sqrt(area));
	}
}

void Graphics::compute_sizes_for_abstract_communities(std::list<Community*> lc, Network* n) {
	std::list<Community*>::iterator lc_it;
	//Calculo los tamaños
	for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
		Community* c = *lc_it;
		c->setRatioAbstract(c->tEdges());
	}
}

void Graphics::generateNetworkFile(std::list<Community*> lc, Network* n, bool abstract) {
	float u = parameters.getU();
	//Uniform probability distribution for edges drawing

	std::list<Community*>::iterator lc_it;

	tColor textColor;

	//Generating mesh for nodes names
	int ppcV = 12; //Pixels per character vertically
	int ppcH = 10; //Pixels per character horizontally

	xMeshSize = round((float)parameters.getWidthResolution()/(ppcH*parameters.getFontZoom()));
	yMeshSize = round((float)parameters.getHeightResolution()/(ppcV*parameters.getFontZoom()));

	namesMesh.resize(xMeshSize);
	for (int i=0;i<xMeshSize;i++) {
		namesMesh[i].resize(yMeshSize);
		for (int j=0;j<yMeshSize;j++) {
			namesMesh[i][j] = 0;
		}
	}

	int i = 0;
	float phiCoordinate = 0;

	std::vector<int>::iterator it_id_vertices;
	std::list<Vertex*>::iterator it_vertices;
	std::list<Vertex*>::iterator vertex_it;
	Vertex* h;

	std::vector<int> id_vertices;
	std::list<Vertex*> vertices;

	float rho_now = 0;
	float r_i1, r_i2, r_j1, r_j2;
	float K; //Constante de la espiral
	float C; //Contante de adaptación entre los tamaños de las comunidades reales y las virtuales (escalado)
	float max1, max2;

	compute_sizes_for_real_communities(lc, n);
	compute_sizes_for_abstract_communities(lc, n);

	lc.sort(CompareCommunitiesSize());
	//lc.sort(CompareCommunitiesSize());

	float sep; //Separación entre comunidades en la espiral
	float beta = 0.5;

	C  = lc.front()->getRatioReal() / lc.front()->getRatioAbstract(); 
	r_i1 = lc.front()->getRatioReal(); //Saving previous ratio for next communities
	r_i2 = lc.front()->getRatioAbstract() * C; //Saving previous ratio for next communities

	//Calculo las coordenadas de cada comunidad
	for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
		Community* c = *lc_it;

//		std::cout << "------------------------------" << std::endl;
//		std::cout << "Radio necesario: " << c->getRatio() << std::endl;
		if (i!=0) {
			if (c->getRatioReal()>c->getRatioAbstract() * C) max1 = c->getRatioReal(); else max1 = c->getRatioAbstract() * C;
			if (r_i1>r_i2) max2 = r_i1; else max2 = r_i2;
			sep = 1.5 * (max1 + max2);
			
			rho_now = calcularRho(K, rho_now, max2, max1, sep, beta);
			//std::cout << "Rho: " << rho_now << std::endl;
			//std::cout << "Phi: " << pow(rho_now/K, (float)1/beta) << std::endl;
		}
		else {
			if (c->getRatioReal()>c->getRatioAbstract() * C) rho_now = c->getRatioReal(); else rho_now = c->getRatioAbstract() * C;
			max1 = rho_now;
//			std::cout << "max1: " << max1 << std::endl;
			if (r_i1>r_i2) max2 = r_i1; else max2 = r_i2;
//			std::cout << "max2: " << max2 << std::endl;
			K = pow((float)2/pi, beta) * 1.5 * (max1 + max2);
//			std::cout << "**** K = " << K << "****" << std::endl;
//			std::cout << "RhoA: " << rho_now << std::endl;
//			std::cout << "PhiB: " << pow(rho_now/K, (float)1/beta) << std::endl;
		}

		c->setPhiCoordinate(pow(rho_now / K, (float)1/beta));
		c->setRhoCoordinate(rho_now);
		c->setX(c->getRhoCoordinate() * cos(c->getPhiCoordinate()));
		c->setY(c->getRhoCoordinate() * sin(c->getPhiCoordinate()));
		r_i1 = c->getRatioReal(); //Saving previous ratio for next communities
		r_i2 = c->getRatioAbstract() * C; //Saving previous ratio for next communities
		i++;
	}
//	std::cout << "Ultimo Rho: " << rho_now << std::endl;

	float ratio = rho_now;
	engine->addHeaders(-1, u, ratio);

	if (!abstract) {
		//REAL - REAL - REAL - REAL - REAL - REAL - REAL - REAL - REAL
		//Para cada comunidad
		i=0;
		for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
			Community* c = *lc_it;
			i++;

			//Aplico el color a todos sus vértices
			tColor n_color = computeHostColor(min(lc.size(), 20), i % 20);

			if (parameters.getBckGnd()=="black")
				textColor = white;
			else
				textColor = black;

			//DESCOMPOSICION EN K-NUCLEOS
			Network* nc = new Network();
			id_vertices = c->getVertices();
			for (it_id_vertices=id_vertices.begin();it_id_vertices!=id_vertices.end();it_id_vertices++) {
				nc->addVertex(n->getVertex(*it_id_vertices));
			}
			//std::cout << "Comunidad de " << n->tVertices() << " vertices y " << n->tEdges() << " ejes" << std::endl;
			nc->buildDegreesList();
			nc->findCores();
			Vertex* v_nc;
			//Copio los resultados del k-core en cada vertice de la comunidad, y del k-core máximo y las propiedades de la comunidad
			vertices = nc->getVertices();
			for (it_vertices=vertices.begin();it_vertices!=vertices.end();it_vertices++) {
				v_nc = n->getVertex((*it_vertices)->getNumber());
				v_nc->setShellIndex((*it_vertices)->getShellIndex());
			}
			c->setMaxShellIndex(nc->getMaxShellIndex());
			//n->findComponents();
			//n->findCoordinates();
			//n->logCoordinates();
			//n->buildClustersMap();

			float nucleus_area = 0;
			std::list<Vertex*> list_top_core = nc->getTopCore();
			for (vertex_it=list_top_core.begin();vertex_it!=list_top_core.end();vertex_it++) {
				int i = (*vertex_it)->getNumber();
				h = n->getVertex(i);
				nucleus_area += pow(h->getRatio(), 2);
			}

			//GRAFICO VERTICES Y EJES
			nucleus_area /= pow(c->getRatioReal(), 2); //Proporción entre el núcleo y el plasma
			nucleus_area *= S;
			//engine->addCircle(c->getRhoCoordinate() * cos(c->getPhiCoordinate()), c->getRhoCoordinate() * sin(c->getPhiCoordinate()), sqrt(nucleus_area) * c->getRatioReal(), n_color);

	//		std::cout << "nucleus_area: " << nucleus_area << std::endl;
	//		float nucleus_area = sqrt((float)nc->getTopCore().size()/c->getSize()); //Proporción entre el núcleo y el plasma
			id_vertices = c->getVertices();
			for (it_id_vertices=id_vertices.begin();it_id_vertices!=id_vertices.end();it_id_vertices++) {
				int i = *it_id_vertices;
				h = n->getVertex(i);
				h->setColor(n_color);

				Uniform unif_nucleus(0, sqrt(nucleus_area));
				Uniform unif_plasma(sqrt(nucleus_area) + 0.2, 1);
				Uniform unif_phi(0, 2*pi);
				float rho, phi;
				if (h->getShellIndex()==c->getMaxShellIndex())
					rho = unif_nucleus.getValue();
				else
					rho = unif_plasma.getValue();
				phi = unif_phi.getValue();
				h->setX(rho*cos(phi));
				h->setY(rho*sin(phi));
//				std::cout << "X: " << h->getX() << std::endl;
				h->setRatio(computeHostRatio(n->getMaxDegree(), (h->getStrength()), (h->getDegree()), (parameters.getWeighted() && (!parameters.getMultigraph()))));

				//std::cout << "X: " << h->getX() << std::endl;
				//std::cout << "Y: " << h->getY() << std::endl;
				translate(h, c->getRhoCoordinate(),  c->getPhiCoordinate(),  c->getRatioReal(), ratio);
	//			std::cout << "X: " << h->getX() << std::endl;
			}
		}

		//External Edges
		for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
			Community* c = *lc_it;
			Vertex* h;
			std::vector<int>::iterator vertex_id_it;
			for (vertex_id_it=c->getVertices().begin();vertex_id_it!=c->getVertices().end();vertex_id_it++) {
				int i = *vertex_id_it;
				h = n->getVertex(i);
				drawEdge(h, u, ratio, n, false, true);
			}
		}
		//Internal Edges
		for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
			Community* c = *lc_it;
			Vertex* h;
			std::vector<int>::iterator vertex_id_it;
			for (vertex_id_it=c->getVertices().begin();vertex_id_it!=c->getVertices().end();vertex_id_it++) {
				int i = *vertex_id_it;
				h = n->getVertex(i);
				drawEdge(h, u, ratio, n, true, false);
			}
		}
		for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
			Community* c = *lc_it;
			Vertex* h;
			std::vector<int>::iterator vertex_id_it;
			for (vertex_id_it=c->getVertices().begin();vertex_id_it!=c->getVertices().end();vertex_id_it++) {
				int i = *vertex_id_it;
				h = n->getVertex(i);
				drawNode(h, u);
			}
		}
		for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
			Community* c = *lc_it;
			Vertex* h;
			std::vector<int>::iterator vertex_id_it;
			for (vertex_id_it=c->getVertices().begin();vertex_id_it!=c->getVertices().end();vertex_id_it++) {
				int i = *vertex_id_it;
				h = n->getVertex(i);
				drawText(h, u, textColor, ppcV, ppcH, ratio, n);
			}
		}


		//Degree references, 5 at most
		if (parameters.getBckGnd()=="black")
			textColor=white;
		else
			textColor=black;

		tColor sphereColor;
		if (parameters.getBckGnd()=="black")
			sphereColor=white;
		else
			sphereColor=gray;
		
		float nc = 15;

		float x = -u * ratio;
	
		if ((parameters.getWeighted()==false)||(parameters.getMultigraph()==true)) {
			float separationRatio = 1.5 * (0.0007 + 0.029 * (log(1 + n->getMaxDegree()) / log(n->getMaxDegree())) * (15));
			separationRatio = max(ratio/25, separationRatio);
			for (int i=1;i<=5;i++) {
				int degree = (int)ceil((float)n->getMaxDegree()/pow(4.0, i-1));
				if (degree > 1) {
					float ratioHost = computeHostRatio(n->getMaxDegree(), 0.0, degree, false);
					float y = -u * ratio * 0.6 + 3.0 * separationRatio * i;
					engine->addHost(x, y, 0, ratioHost, sphereColor, sphereColor);
					std::stringstream strm;
					strm << degree;
					std::string shellNumber = strm.str();
					float size = 2.0 * separationRatio;
					engine->addText(shellNumber, textColor, size, ((x + separationRatio) / size), ((y - 1.5 * separationRatio) / size), "left");
				}
			}
		} else {
			float separationRatio = 1.5 * (0.0007 + 0.029 * (log(1 + n->getMaxStrength()) / log(n->getMaxStrength()))) * (15);
			for(int i=1;i<=5;i++) {
				float strength = n->getMaxStrength()/pow(4.0,(i-1));
			
				float ratio = computeHostRatio(n->getMaxDegree(), strength, 0, true);
				float y = -u * nc * 0.5 + 3.0 * separationRatio * i;
		
				engine->addHost(x, y, 0, ratio, sphereColor, sphereColor);
	
				std::stringstream strm;
				strm << strength;
				std::string shellNumber = strm.str();

				float size = 2.0 * separationRatio;
				engine->addText(shellNumber, textColor, size, ((x + separationRatio) / size), ((y - 0.5 * separationRatio) / size), "left");
	
			}
		}
	}
	else {
		//ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT - ABSTRACT
		//Para cada comunidad
		std::vector<Community*> neighbours;
		std::vector<Community*>::iterator neighbours_it;
		std::vector<unsigned int> neighbour_cuts;
		std::vector<unsigned int>::iterator neighbour_cuts_it;
		i=0;
		for (lc_it = lc.begin(); lc_it != lc.end(); lc_it++) {
			Community* c = *lc_it;
			i++;
			tColor n_color = computeHostColor(min(lc.size(), 20), i % 20);
			engine->addHost(c->getRhoCoordinate() * cos(c->getPhiCoordinate()), c->getRhoCoordinate() * sin(c->getPhiCoordinate()), 0, c->getRatioAbstract() * C, n_color, n_color);
			neighbours = c->getNeighbourCommunities();
			neighbour_cuts = c->getNeighbourCommunityCuts();
			neighbour_cuts_it=neighbour_cuts.begin();
			for ((neighbours_it=neighbours.begin());neighbours_it!=neighbours.end();(neighbours_it++)) {
				engine->addHost(c->getX(), c->getY(), 0, (*neighbour_cuts_it)*C, white, white, parameters.getOpacity());
				engine->addHost((*neighbours_it)->getX(), (*neighbours_it)->getY(), 0, (*neighbour_cuts_it)*C, white, white, parameters.getOpacity());
				engine->addCylinder(c->getX(), c->getY(), 0, (*neighbours_it)->getX(), (*neighbours_it)->getY(), 0, white, parameters.getOpacity(), (*neighbour_cuts_it)*C);
				neighbour_cuts_it++;
			}
		}

		//Grafico la espiral
		/*float angle = 0;
		float rho = 0;
		float angle_new, rho_new;
		do {
			angle_new = angle + 2 * pi / 100;
			rho_new = K*pow(angle_new, beta);
			engine->addCylinder(rho*cos(angle), rho*sin(angle), 0, rho_new*cos(angle_new), rho_new*sin(angle_new), 0, white, parameters.getOpacity(), (*lc.begin())->getRatioAbstract() * 0.02 * C);
			rho = rho_new;
			angle = angle_new;
		} while (rho < rho_now);
		*/
	}
		
	engine->close();

}

