#include <iostream>
#include <fstream>
#include "graph_builder_txt.h"
#include "graph_node_names.h"
#include "network.h"
#include "stdlib.h"
#include "parameters.h"
#include "engine.h"
#include "povray.h"
#include "svg.h"
#include "graphics.h"
#include "povray_renderer.h"
#include "svg_renderer.h"
#include <vector>
#include <algorithm>
#include "pair_communities.h"
#include <fstream>

int main(int argc, char* argv[]) {

	Graph_builder_txt graph_builder;
	std::vector<Network*> comm_graphs; //Community graphs
	parameters.validateParameters(argc, argv);

	std::ifstream file;
	file.open(parameters.getInputFile().c_str(), std::fstream::in);

	Network* graph = new Network();
	std::cout << "\tBuilding graph...\n";
	graph_builder.readFile(file, *graph);
	//graph.printGraph();

	std::cout << "\tFinding communities...\n";
	
	graph->findCores();

	graph->findCommunities(1);
	std::list<Community*>::iterator it_communities;
	std::list<Community*> communities = graph->getCommunities();
	std::vector<int>::iterator it_vertices;

	int total_nodos = 0;

	//std::cout << " " << communities.size() << " communities, " << total_nodos << " nodes in top cores, " << graph->tVertices() << " nodes, " << graph->tEdges() << " edges" << ", Q=" << graph->getQ() << std::endl;

	//Generating image
	Engine *e, *e2;

	if (parameters.getRenderer()=="povray") {
		std::cout << "\tGenerating .pov file...\n";
		e = new Povray(parameters.getOutputType1File());
		e2 = new Povray(parameters.getOutputType2File());
	}
	else {
		std::cout << "\tGenerating .svg file...\n";
		e = new Svg(parameters.getOutputType1File());
		e2 = new Svg(parameters.getOutputType2File());
	}
	Graphics graphics(e);
	graphics.generateNetworkFile(comm_graphs);
	graphics.generateNetworkFile(communities, graph, false); //Real
	Graphics graphics2(e2);
	graphics2.generateNetworkFile(communities, graph, true); //Abstract

	if (parameters.getNoGraphic()==false) {
		//Render visualization
		std::cout << "\tRendering image...\n";
		if (parameters.getRenderer()=="povray") {
			povray_renderer.render(parameters.getOutputType1File(), parameters.getOutputPngType1File());
			povray_renderer.render(parameters.getOutputType2File(), parameters.getOutputPngType2File());
		}
		else {
			svg_renderer.render(parameters.getOutputType1File(), parameters.getOutputPngType1File());
			svg_renderer.render(parameters.getOutputType2File(), parameters.getOutputPngType2File());
		}
	}

	file.close();
	comm_graphs.clear();

	std::cout << "Done!\n";

}

