#include "network.h"
#include <iostream>
#include "types.h"
#include "solution_submodular.h"
#include "math.h"
#include "uniform.h"
#include <algorithm>
#include "parameters.h"

Network::Network() : mainComponent(0, NULL) {
	maxShellIndex = 0;
	minShellIndex = 0;
	aux = false;
}

Network Network::getInducedGraph(std::list<Vertex*> vertices) {
	Network new_graph;
	std::list<Vertex*>::iterator it = vertices.begin();
	while (it!=vertices.end()) {
		new_graph.addVertex(*it);
		it++;
	}
	return new_graph;
}

Network Network::getLayer(int layer) {
	Network new_graph;
	std::list<Vertex*>::iterator it = vertices.begin();
	while (it!=vertices.end()) {
		if ((*it)->getShellIndex()>=layer)
			new_graph.addVertex(*it);
		it++;
	}
	return new_graph;
}

std::list<Vertex*> Network::getTopCore() {
	return coresList[maxShellIndex];
}

void Network::findCores() {
	coresList = degreesList;

	std::list<Vertex*>::iterator it;
	for (it=vertices.begin();it!=vertices.end(); it++) {
		(*it)->setShellIndex((*it)->getDegree());
	}

	Vertex* v;
	std::vector<Vertex*>* v_neighbours;

	std::map<int, std::list<Vertex*> >::iterator list_it;
	std::list<Vertex*>::iterator vertex_it;
	std::vector<Vertex*>::iterator neighb_it;

	int actualShellIndex;

	//For each list
	for (list_it=coresList.begin();list_it!=coresList.end(); list_it++) {
		actualShellIndex = list_it->first;
		//For each vertex
		for (vertex_it=list_it->second.begin();vertex_it!=list_it->second.end(); vertex_it++) {
			v=*vertex_it;
			v_neighbours = v->getNeighbours();
			//For each neighbour
			for (neighb_it=v_neighbours->begin();neighb_it!=v_neighbours->end(); neighb_it++) {
				std::list<Vertex*>* core;
				int shellIndex = (*neighb_it)->getShellIndex();
				if (shellIndex > actualShellIndex) {
					//Removing from previous core
					core = &coresList[shellIndex];
					std::list<Vertex*>::iterator it = find(core->begin(), core->end(), (*neighb_it));
					core->erase(it);
					if (core->size() == 0)
						coresList.erase(shellIndex);
					//Moving to a former core
					(*neighb_it)->setShellIndex(shellIndex-1);
					coresList[shellIndex-1].push_back(*neighb_it);
				}			
			}
		}
	}

	//Find maximum and minimum shell index
	std::map<int, std::list<Vertex*> >::iterator map_it;
	map_it = coresList.begin();
	minShellIndex = map_it->first;
		while (map_it!=coresList.end()) {
			maxShellIndex = map_it->first;
			map_it++;
		}
}

void Network::findComponents() {
	mainComponent.computeComponents(&vertices, maxShellIndex, minShellIndex, maxDegree);
}

void Network::buildClustersMap() {

	mainComponent.buildClustersMap(clustersMap);

	/*std::map<int, std::list<std::list<Vertex*>* > >::iterator map_it;
	for (map_it=clustersMap.begin();map_it!=clustersMap.end();map_it++) {
		std::cout << "k = " << map_it->first << std::endl;
		std::list<std::list<Vertex*>* > clusters = map_it->second;
		std::list<std::list<Vertex*>* >::iterator clusters_it;
		for (clusters_it=clusters.begin();clusters_it!=clusters.end();clusters_it++) {
			std::list<Vertex*>* cluster = *clusters_it;
			std::list<Vertex*>::iterator cluster_it;
			std::cout << "    ";
			for (cluster_it=cluster->begin();cluster_it!=cluster->end();cluster_it++) {
				std::cout << (*cluster_it)->getNumber() << " ";
			}
			std::cout << std::endl;
		}
	}*/
}

int intersection(std::vector<Vertex*>* l1, std::vector<Vertex*>* l2) {
	std::vector<Vertex*>::iterator it1 = l1->begin();
	int i = 0;
	it1 = std::find_first_of(it1, l1->end(), l2->begin(), l2->end());
	while (it1 != l1->end()) {
		i++;
		it1++;
		it1 = std::find_first_of(it1, l1->end(), l2->begin(), l2->end());
	}
	
	return i;
}

void Network::printComponents() {
	mainComponent.printComponent();
}

void Network::printCores() {

	Vertex* v;

	std::map<int, std::list<Vertex*> >::iterator list_it;
	std::list<Vertex*>::iterator vertex_it;

	std::cout << "Cores BEGIN" << std::endl;
	//For each list
	for (list_it=coresList.begin();list_it!=coresList.end(); list_it++) {
		std::cout << list_it->first << ": ";
		//For each vertex
		for (vertex_it=list_it->second.begin();vertex_it!=list_it->second.end(); vertex_it++) {
			v=*vertex_it;
			std::cout << v->getNumber() << " ";
		}
		std::cout << std::endl;
	}
	std::cout << "Cores END" << std::endl;
}

void Network::logCores() {

	Vertex* v;

	std::map<int, std::list<Vertex*> >::iterator list_it;
	std::list<Vertex*>::iterator vertex_it;

	//For each list
	for (list_it=coresList.begin();list_it!=coresList.end(); list_it++) {
		//For each vertex
		for (vertex_it=list_it->second.begin();vertex_it!=list_it->second.end(); vertex_it++) {
			v=*vertex_it;
			logCore << list_it->first << " " << v->getNumber() << "\n";
		}
	}
}

//COMMUNITIES

void Network::printCommunities() {
	std::list<Community*>::iterator comm_it;
	int comm_number=1;
	for (comm_it = communities.begin(); comm_it != communities.end() ; comm_it++) {
		if ((*comm_it)->removed == false) {
			(*comm_it)->printCommunity();
/*			std::cout << comm_number << ": ";
			std::vector<Vertex*>::iterator v_it;
			for (v_it = (*comm_it)->getVertices().begin(); v_it != (*comm_it)->getVertices().end() ; v_it++) {
				std::cout << (*v_it)->getNumber() << " ";
			}
			std::cout << std::endl;
			comm_number++;
*/		}
	}
}

void Network::logAllCommunities() {
	logCommunities << "#Number; Size; e_ii; a_ij; Sum degrees;\n";
	std::list<Community*>::iterator comm_it;
	int comm_number=1;
	for (comm_it = communities.begin(); comm_it != communities.end() ; comm_it++) {
		if ((*comm_it)->removed == false) {
			logCommunities << comm_number << " " << (*comm_it)->getSize() << " " << (int)(*comm_it)->tEdges() << " " << (int)((*comm_it)->tDegrees() - (*comm_it)->tEdges()) << " " << (int)(*comm_it)->tDegrees() << "\n";
			std::vector<int>::iterator v_it;
//			for (v_it = (*comm_it)->getVertices().begin(); v_it != (*comm_it)->getVertices().end() ; v_it++) {
//				logCommunities << comm_number << " " << (*v_it) << " " << "\n";
//			}
			comm_number++;
		}
	}
}

void Network::assignCommunitiesFromFile(std::string filename) {
/*	std::ifstream f;
	int n1,n2;
	std::map<int, Community*> mapita;
	f.open(filename.c_str());
	f >> n1 >> n2;
	while (!f.eof()) {
		//std::cout << n1 << " " << n2 << std::endl;
		if (mapita.find(n2)==mapita.end()) {
			Community* c = new Community();
			mapita[n2] = c;
			communities.push_back(c);
		}
		vertices_map[n1]->setCommunity(mapita[n2]);
		mapita[n2]->addVertex(vertices_map[n1]);
		f >> n1 >> n2;
	}
	f.close();
*/}

//Find communities up to t
void Network::findCommunities(float t_final) {

	//Armado inicial de comunidades
	std::vector<Community*>::iterator it;
	int a = 0;
	for (it=existent_vertices.begin();it!=existent_vertices.end();it++) {
		if ((*it) != NULL) {
			communities.push_back((*it));
		}
	}
	bool unaSolaComm=false;

	SolutionSubmodular* solution = new SolutionSubmodular(communities, this);
	unaSolaComm = !solution->proposeNeighbour();

	float t = solution->evaluateNeighbour();

	//std::cout << "t: " << t << std::endl;
	float Q=-5;
	float Qant=-10;
	float Qt;
//	while (Q >= Qant) {
//	while ((t >= 1)/*||(Q>Qant)*/) {
	int i=0;
	while ((t >= t_final)&&(!unaSolaComm)) {
		i++;
		/*if ((i % int(pow(10,floor(log(i)/log(10))))) == 0)
			std::cout << "Paso " << i << std::endl;
		Qant = Q;
		*/solution->acceptNeighbour();
		//Q = solution->evaluate();
		//Qt = solution->getQt();
		//printCommunities();
		//std::cout << t << std::endl;
		//std::cout << "t: " << t << ", Q seguro: " << getQ() << std::endl << std::endl;
//		if ((i%20)==0)
//			logKMax << t << " " << getKMax() << "\n";
		//logKMin << t << " " << getKMin() << "\n";
		unaSolaComm = !solution->proposeNeighbour();
		t = solution->evaluateNeighbour();

	}
	finalizeCommunities();
	//std::cout << "T: " << t << std::endl;
	//printCommunities();
	//std::cout << std::endl;
	//std::cout << "Q: " << solution->evaluate() << std::endl << std::endl;
	//std::cout << "1 " << getQ() << std::endl;
	logAllCommunities();
		
/*	Solution* solution = new SolutionKCores(vert, communities, partial_graph);
	std::cout << "Q final: " << solution->evaluate() << std::endl;
*/
}

void Network::finalizeCommunities() {
	std::list<Community*>::iterator comm_it;
	int comm_number=1;
	for (comm_it = communities.begin(); comm_it != communities.end() ; comm_it++) {
		if ((*comm_it)->removed == false) {
			std::vector<int>::iterator v_it;
			for (v_it = (*comm_it)->getVertices().begin(); v_it != (*comm_it)->getVertices().end() ; v_it++) {
				getVertex(*v_it)->setCommunity(*comm_it);
			}
			comm_number++;
		}
	}
}
float Network::getQ() {
	float Q = 0;
	float Qnum1 = 0, Qnum2 = 0;
	std::list<Community*>::iterator comm_it;
	Community* C;
	for (comm_it = communities.begin(); comm_it != communities.end() ; comm_it++) {
		C = *comm_it;
		if ((C->removed == false)&&(C->tEdges()!=0)) {
			Qnum1 += C->tEdges();
			Qnum2 += (float)C->tDegrees() * C->tDegrees();
		}
	}
	Q = (Qnum1 - Qnum2/(total2Edges)) / (total2Edges);
	return Q;
}

