#ifndef _NETWORK_H_
#define _NETWORK_H_

#include "graph.h"
#include "component.h"
#include "ff_routine.h"

typedef struct {
	std::list<Vert*> lista_num;
	std::list<void*> hijos;
	std::list<int> flujos;
	void* padre;
	int flujo_padre;
	int numero_de_hijo;
} Nodo;

typedef struct {
	Nodo* raiz;
} Arbol;

class Network : public Graph {
  private:

	Arbol gomory_hu_tree;

	//Component of core 0
	Component mainComponent;

	//Maximum shell index. Also called cmax
	int maxShellIndex;

	//Minimum shell index.
	int minShellIndex;

	//Map of vertices of the same core
	std::map<int, std::list<Vertex*> > coresList;

	std::list<Community*> communities;

  public:
	std::map<int, std::list<std::list<Vertex*>* > > clustersMap;

	std::list<Component*> inline getComponents() { return mainComponent.getComponents(); };
	Component* getMainComponent() { return &mainComponent; };
	std::list<Vertex*> getTopCore();
	Network();

	Network getInducedGraph(std::list<Vertex*> vertices);

	//Gets the induced graph obtained from a core from the k-core decomposition
	Network getLayer(int layer);

	void findCores();
	void printCores();
	void logCores();

	void findComponents();
	void printComponents();

	void buildClustersMap();

	inline std::list<Community*>& getCommunities() { return communities; }
	void findCommunities(float t_final = 1);
	void finalizeCommunities();
	void assignCommunitiesFromFile(std::string filename);
	void printCommunities();
	void logAllCommunities();

	float getQ();
	float getKMax();
	float getKMin();
	void calcularCota();
	int verifySubmodularity();

	inline int getMaxShellIndex() { return maxShellIndex;}
	inline void setMaxShellIndex(int shInd) { maxShellIndex = shInd;}
	inline int getMinShellIndex() { return minShellIndex;}
	inline void setMinShellIndex(int shInd) { minShellIndex = shInd;}

	void build_gomory_hu_tree(int No_of_Node);

	int find_connectivity(int v1, int v2);

	void find_connectivities();

	void find_inner_connectivities(int layer);

	int find_minimum_cut();

	void find_path(int v1, std::list<std::pair<int, int> >& camino1);

	void print_gomory_hu_tree(Vert* vertices, bool archivo=true);

	bool aux;
	//int verifySubmodularity();
};

#endif

