#include "svg.h"
#include "parameters.h"

Svg::Svg(std::string filename) {
	out_file.open(filename.c_str(), std::fstream::out);
	r_saved = 0.0;
	u_saved = 0.0;
}
	
void Svg::addHeaders(int nc, float u, float r) {
	out_file << "<?xml version=\"1.0\" standalone=\"no\"?>\n";
	out_file << "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n";
  	out_file << "\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n";

	float ratio = u * 2.0 * 1.3 * r;
	float width = 1.2 * ratio;
	float height = 0.9 * ratio;
	out_file << "<svg width=\"800\" height=\"600\" viewBox=\"" << (-0.6*ratio + parameters.getHStart() * width) << " " << (-0.45*ratio + parameters.getVStart() * height) << " " << (width * (parameters.getHEnd() - parameters.getHStart())) << " " << (height*(parameters.getVEnd() - parameters.getVStart())) << "\" version=\"1.1\"\n";
	out_file << "xmlns=\"http://www.w3.org/2000/svg\">\n";
	if (parameters.getBckGnd() != "white")
		out_file << "<rect x=\"-" << (0.6*ratio) << "\" y=\"-" << (0.45*ratio) << "\" width=\"" << (1.2*ratio) << "\" height=\"" << (0.9*ratio) << "\" fill=\"rgb(0,0,0)\"/>\n";
}

void Svg::addHost(float x, float y, float z, float ratio, tColor color, tColor borderColor, float opacity) {

	float x1 = x;
	float x2 = x;
	float y1 = y;
	float y2 = y;
	float diameter = ratio * 1.5;
	float z1 = z + ratio + 0.1;
	float z2 = z + ratio + 0.2;

	if ((color.r != borderColor.r)||(color.g != borderColor.g)||(color.b != borderColor.b)) {
		out_file << "<circle cx=\"" << x << "\" cy=\"" << -y << "\" fill=\"rgb(" << round(borderColor.r * 255) << "," << round(borderColor.g * 255) << "," << round(borderColor.b * 255) << ")\" r=\"" << ratio * 1.10 << "\" opacity=\"" << opacity << "\"/>\n";
	}

	out_file << "<circle cx=\"" << x << "\" cy=\"" << -y << "\" fill=\"rgb(" << round(color.r * 255) << "," << round(color.g * 255) << "," << round(color.b * 255) << ")\" r=\"" << ratio << "\" opacity=\"" << opacity << "\"/>\n";

}

void Svg::addBlock(float x, float y, float z, float ratio, tColor color) {
	out_file << "<rect x=\"" << (x - ratio) << "\" y=\"" << (-y + ratio) << "\" fill=\"rgb(" << round(color.r * 255) << "," << round(color.g * 255) << "," << round(color.b * 255) << ")\" width=\"" << (2 * ratio) << "\" height=\"" << (2 * ratio) << "\"/>\n";
}

void Svg::addCylinder(float x1, float y1, float z1, float x2, float y2, float z2, tColor color, float opacity, float diameter) {
	//Verify that base point != apex point, for that would generate an error in povray
	if ((floor((x1+0.0005)*1000) != floor((x2+0.0005)*1000)) || (floor((y1+0.0005)*1000) != floor((y2+0.0005)*1000)))
		out_file << "<line x1=\"" << x1 << "\" x2=\"" << x2 << "\" y1=\"" << (-y1) << "\" y2=\"" << (-y2) << "\" stroke=\"rgb(" << round(color.r * 255) << "," << round(color.g * 255) << "," << round(color.b * 255) << ")\" stroke-width=\"" << (diameter*2) << "\" opacity=\"" << opacity << "\"/>\n";
}

void Svg::addText(std::string text, tColor color, float size, float x, float y, std::string align) {
	if (align=="left")
		out_file << "<text x=\"" << (x*size) << "\" y=\"" << (-y*size-0.65*size) << "\" font-family=\"Verdana\" font-size=\"" << size << "\" fill=\"rgb(" << round(color.r * 255) << "," << round(color.g * 255) << "," << round(color.b * 255) << ")\">" << text.c_str() << "</text>\n";
	else
		out_file << "<text x=\"" << (x*size) << "\" y=\"" << (-y*size-0.65*size) << "\" font-family=\"Verdana\" font-size=\"" << size << "\" fill=\"rgb(" << round(color.r * 255) << "," << round(color.g * 255) << "," << round(color.b * 255) << ")\">" << text.c_str() << "</text>\n";
}

void Svg::addCircle(float x, float y, float ratio, tColor color) {
	out_file << "<circle cx=\"" << x << "\" cy=\"" << -y << "\" border=\"rgb(" << round(color.r*255) << "," << round(color.g*255) << "," << round(color.b*255) << ")\" r=\"" << ratio << "\" fill=\"rgb(60,40,0)\"/>\n";
}

void Svg::close() {
	out_file << "</svg>\n";
	//Close file*)
	out_file.close();
}

