#include "vertex.h"
#include <algorithm>
#include <iostream>

Vertex::Vertex(int m_number) {
	mark = 0;
	number = m_number;
	degree = 0;
	community = NULL;

	rho = -1.0;
	phi = -1.0;
	coordinatesComputed = false;

	x = 0.0;
	y = 0.0;
	ratio = 0.0;
	color.r = 0.0;
	color.g = 0.0;
	color.b = 0.0;
	isKConnected = false;
	clusteringCoefficient = 0.0;
	cluster = NULL;
	kConnectivity = 0;
}

Vertex::Vertex(Vertex* v) {
	mark = 0;
	number = v->number;
	degree = 0;
	community = 0;

	rho = v->rho;
	phi = v->phi;
	coordinatesComputed = v->coordinatesComputed;

	x = v->x;
	y = v->y;
	ratio = v->ratio;
	color = v->color;
	isKConnected = false;
	clusteringCoefficient = 0;
	cluster = 0;
	kConnectivity = 0;
}

void Vertex::addNeighbour(Vertex* v) {
	neighbours.push_back(v);
	w.push_back(1);
	degree++;
}

bool Vertex::hasNeighbour(Vertex* v) {
	std::vector<Vertex*>::iterator nb_it;
	nb_it = find(neighbours.begin(), neighbours.end(), v);
	if (nb_it != neighbours.end())
		return true;
	return false;
}

int Vertex::getDegree() {
	return degree;
}

float Vertex::getWeight(int i) {
	return w[i];
}

float Vertex::getStrength() {
	float s = 0.0;
	for (int i=0;i<neighbours.size();i++) {
		s += w[i];
	}
	return s;
}

void Vertex::setShellIndex(int m_shellIndex) {
	shellIndex = m_shellIndex;
}

void Vertex::setComponent(Component* m_component) {
	component = m_component;
}

void Vertex::setCluster(std::list<Vertex*>* m_cluster) {
	cluster = m_cluster;
}

/*void Vertex::addCommunityCut(Community* c) {
	std::map<Community*, int>::iterator it;
	it = community_cuts.find(c);
	if (it != community_cuts.end())
		it->second = it->second + 1;
	else
		community_cuts[c] = 1;
}*/

/*void Vertex::removeCommunityCut(Community* c) {
	int cut = community_cuts[c];
	if (cut != 1) {
		community_cuts[c] -= 1;
	}
	else
		community_cuts.erase(c);
}*/

/*int Vertex::cut(Community* c) {
	return community_cuts[c];
}
*/
