#ifndef _VERTEX_H_
#define _VERTEX_H_

#include <list>
#include <vector>
#include <map>
#include "component.h"
#include "community.h"
#include "types.h"
#include <sstream>

class Component;
class Community;

class Vertex {
  private:
	int number;

	int degree;
	int shellIndex;

	int mark;

	float rho, phi, x, y, ratio;
	float clusteringCoefficient;
	tColor color, bordercolor;
	bool isKConnected;

	Component* component;
	Community* community;
	std::list<Vertex*>* cluster;
	std::vector<Vertex*> neighbours;
	std::vector<int> w;

	int kConnectivity;
	bool coordinatesComputed;

	std::string name;
	//std::map<Community*, int> community_cuts;

  public:
	Vertex(int m_number);
	Vertex(Vertex* v);
	void addNeighbour(Vertex* v);
	bool hasNeighbour(Vertex* v);
	inline std::vector<Vertex*>* getNeighbours() { return &neighbours; }
	inline int getNumber() { return number; }
	int getDegree();
	float getWeight(int i);
	float getStrength();
	
	void setShellIndex(int m_shellIndex);
	inline int getShellIndex() {return shellIndex; }

	inline void setMark(int m_mark) {mark = m_mark; }
	inline int getMark() {return mark; }

	void setComponent(Component* m_component);
	inline Component* getComponent() {return component; }

	void setCluster(std::list<Vertex*>* m_cluster);
	inline std::list<Vertex*>* getCluster() {return cluster;}

	inline void setCommunity(Community* m_community) { community = m_community; }
	inline Community* getCommunity() {return community;}

	inline void setRho(float r) { rho = r;}
	inline void setPhi(float p) { phi = p;}
	inline void setX(float xi) { x = xi;}
	inline float setY(float yi) { y = yi;}
	inline float setRatio(float r) { ratio = r;}
	inline float getRho() { return rho;}
	inline float getPhi() { return phi;}
	inline float getX() { return x;}
	inline float getY() { return y;}
	inline float getRatio() { return ratio;}
	inline void setClusteringCoefficient(float cc) { clusteringCoefficient = cc;}
	inline float getClusteringCoefficient() { return clusteringCoefficient;}
	
	inline void setColor(tColor c) { color = c;}
	inline tColor getColor() { return color;}
	inline void setBorderColor(tColor c) { bordercolor = c;}
	inline tColor getBorderColor() { return bordercolor;}

	int getKConnectivity() { return kConnectivity; }
	void setKConnectivity(int c) { kConnectivity = c; }

	bool getIsKConnected() { return (kConnectivity!=0); }
	bool getIsInC() { return (kConnectivity==shellIndex); }
	bool getIsInD() { return (kConnectivity!=0); }

	inline void setName(std::string n) { name = n; }
	inline std::string getName() { return name; }
	inline bool getCoordinatesComputed() { return coordinatesComputed; }
	inline void setCoordinatesComputed(bool c) { coordinatesComputed = c; }
	//void addCommunityCut(Community* c);
	//void removeCommunityCut(Community* c);
	//int cut(Community* c);
};

#endif

